// long_double_mask.cpp

// Copyright (c) 2006 Johan Rade

// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt
// or copy at http://www.boost.org/LICENSE_1_0.txt)

#include "fpclassify.hpp"
#include <limits>

namespace boost {
namespace math {
namespace detail {

//------------------------------------------------------------------------------

template<class T>
boost::uint32_t exponent_mask()
{
	T x = std::numeric_limits<T>::infinity();
	boost::uint32_t a;
	copy(x,a);

	if(a == 0x7fff8000)
		// Intel double extended format
		return 0x7fff0000;

	return a;
}

template<class T>
boost::uint32_t significand_mask()
{
	T x = std::numeric_limits<T>::infinity();
	boost::uint32_t a;
	copy(x,a);

	if(a == 0x7fff8000)
		// Intel double extended format
		return 0x00008000;

	return 0x00000000;
}

template<class T>
boost::uint32_t fraction_mask()
{
	T x = -std::numeric_limits<T>::infinity();
	boost::uint32_t a;
	copy(x,a);
	return ~a;
}

//------------------------------------------------------------------------------

const boost::uint32_t mask<long double>::exponent
	= exponent_mask<long double>();

const boost::uint32_t mask<long double>::significand
	= significand_mask<long double>();

const boost::uint32_t mask<long double>::fraction
	= fraction_mask<long double>();

//------------------------------------------------------------------------------

}
}
}


